/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.fhirgateway;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.fhirgw.json.GatewayTargetJson;
import ca.cdr.api.fhirgw.model.ISearchResultsAccumulator;
import ca.cdr.api.fhirgw.model.SearchRequest;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.rest.client.api.IClientInterceptor;
import ca.uhn.fhir.rest.client.api.IHttpRequest;
import ca.uhn.fhir.rest.client.api.IHttpResponse;
import ca.uhn.fhir.rest.server.servlet.ServletRequestDetails;

import java.io.IOException;

/**
 * This interceptor is intended to be used in a FHIR Gateway that is configured with multiple
 * search targets.
 *
 * It modifies the uri used for searches for a specific target.
 * For example, the first target doesn't support usual FHIR endpoint '/_search' when using http POST method,
 * but instead offer the same functionality under '/myCustomSearch', in its FHIR API, for some reason.
 * Note: yep, some FHIR servers use creative endpoints!
 *
 * So in this example, we are simply removing '/_search' from the uri of the target when searching using POST method,
 * and replace it with the expected server endpoint '/myCustomSearch'.
 */

@Interceptor
public class GatewayUpdateTargetUriUsingClientInterceptor {

	@CdrHook(CdrPointcut.FHIRGW_SEARCH_TARGET_PREINVOKE)
	public void hook(SearchRequest theRequest, GatewayTargetJson theTarget, ISearchResultsAccumulator theISearchResultsAccumulator, ServletRequestDetails theRequestDetails) {

		// We assume there are multiple targets, and the first one is called "target1".
		// This logic only applies to the first target, that also uses http POST method for searching,
		// as configured in Gateway target json configuration
		if (theTarget.getId().equals("target1")
			&& theTarget.getUseHttpPostForAllSearches()) {

			// in that case, we want to modify the uri used to call the target server
			// using a client interceptor
			theRequest.setClientInterceptor(new IClientInterceptor() {

				@Override
				public void interceptRequest(IHttpRequest theHttpRequest) {

					if (theHttpRequest.getUri().endsWith("/_search")) {
						// assuming there is no other '/_search' string in the uri,
						// this will make the Gateway FHIR client do this call:
						// POST [baseUrl]/myCustomSearch
						String correctedUri = theHttpRequest.getUri().replace("/_search", "/myCustomSearch");
						theHttpRequest.setUri(correctedUri);
					}
				}

				@Override
				public void interceptResponse(IHttpResponse theIHttpResponse) throws IOException {
					// nothing to do here!
				}
			});
 		}

	}

}
