/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.fhirstorage;

import ca.uhn.fhir.interceptor.api.Hook;
import ca.uhn.fhir.interceptor.api.Pointcut;
import ca.uhn.fhir.rest.api.server.IPreResourceShowDetails;
import ca.uhn.fhir.rest.api.server.RequestDetails;
import com.google.common.collect.Sets;
import org.apache.commons.lang3.tuple.Pair;
import org.hl7.fhir.instance.model.api.IBaseCoding;
import org.hl7.fhir.instance.model.api.IBaseResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.Iterator;
import java.util.Set;

/**
 * This interceptor implements the concept of "reserved tags", which are resource tags (tags in
 * <code>Resource.meta.tag</code>) that are reserved for internal system use. This means that:
 * <ul>
 *    <li>Clients can not add these tags manually</li>
 *    <li>Reserved tags are preserved between updates of resources that have them</li>
 *    <li>The reserved tags are not shown to clients and are filtered before serialization</li>
 * </ul>
 * <p>
 * The assumption with this interceptor is that there is a separate process that manages resource tags.
 * This could be a second interceptor, or some other non-client-facing process.
 * <p>
 * Note that each of the hook methods are annotated with {@literal @Hook(..., order=100)}. The assumption
 * here is that a separate interceptor is managing the reserved tags. That interceptor could use
 * {@literal @Hook(..., order=101)} to ensure that it comes afterward so that it can add or remove
 * the reserved tags as necessary.
 */
@SuppressWarnings("unchecked")
public class TagPreservingInterceptor {

	private static final Logger ourLog = LoggerFactory.getLogger(TagPreservingInterceptor.class);

	private Set<Pair> myTagsToPreserve;

	/**
	 * Constructor
	 */
	public TagPreservingInterceptor() {
		// This is the list of tags we want to preserve
		myTagsToPreserve = Collections.unmodifiableSet(Sets.newHashSet(
			Pair.of("http://my-tags", "tag1"),
			Pair.of("http://my-tags", "tag2")
		));
	}

	/**
	 * When a new resource is being created, don't allow the client to include any reserved tags
	 *
	 * @param theRequest  Contains details about the incoming request. You may wish to cast this
	 *                    object to a {@link ca.uhn.fhir.rest.server.servlet.ServletRequestDetails},
	 *                    which is always possible when running interceptors inside Smile CDR.
	 * @param theResource The previous resource being created
	 */
	@Hook(value = Pointcut.STORAGE_PRESTORAGE_RESOURCE_CREATED, order = 100)
	public void resourcePreCreate(RequestDetails theRequest, IBaseResource theResource) {
		removeReservedTags(theResource);
	}

	/**
	 * For this example we are overriding the <code>resourcePreUpdate</code>
	 * method, which will be called when a resource is being updated. There
	 * are many other methods that can be overridden, but this one is
	 * appropriate for the desired functionality.
	 *
	 * @param theRequest     Contains details about the incoming request. You may wish to cast this
	 *                       object to a {@link ca.uhn.fhir.rest.server.servlet.ServletRequestDetails},
	 *                       which is always possible when running interceptors inside Smile CDR.
	 * @param theOldResource The previous version of the resource, before this update was
	 *                       requested by the client.
	 * @param theNewResource The new version of the resource as submitted by the client. This
	 *                       object can be modified.
	 */
	@Hook(value = Pointcut.STORAGE_PRESTORAGE_RESOURCE_UPDATED, order = 100)
	public void resourcePreUpdate(RequestDetails theRequest, IBaseResource theOldResource, IBaseResource theNewResource) {

		// Filter out any reserved tags
		removeReservedTags(theNewResource);

		// Copy tags forward
		theOldResource
			.getMeta()
			.getTag()
			.stream()
			.filter(t -> myTagsToPreserve.contains(Pair.of(t.getSystem(), t.getCode())))
			.forEach(t -> theNewResource.getMeta().addTag()
				.setSystem(t.getSystem())
				.setCode(t.getCode())
				.setDisplay(t.getDisplay()));
	}

	/**
	 * Before returning any resource to the client, strip the reserved tags
	 */
	@Hook(value = Pointcut.STORAGE_PRESHOW_RESOURCES)
	public void preShow(IPreResourceShowDetails thePreShowDetails) {
		for (IBaseResource nextResource : thePreShowDetails) {
			removeReservedTags(nextResource);
		}
	}

	private void removeReservedTags(IBaseResource theResource) {
		for (Iterator<? extends IBaseCoding> iter = theResource.getMeta().getTag().iterator(); iter.hasNext(); ) {
			IBaseCoding nextTag = iter.next();
			String system = nextTag.getSystem();
			String code = nextTag.getCode();
			if (myTagsToPreserve.contains(Pair.of(system, code))) {
				iter.remove();
			}
		}
	}

}
