/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.smartout;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.fhir.interceptor.OidcAuthRequestDetails;
import ca.uhn.fhir.interceptor.api.Interceptor;
import org.apache.commons.lang3.StringUtils;


// START SNIPPET: interceptor
/**
 * This interceptor decides which server to select for Federated OIDC
 * mode. In this example, we are using the "launch" request parameter
 * sent to us by the client to determine which server to select.
 * The launch parameter is provided by the EHR to the Client with
 * the intent that the Client will return it unmodified. We assume
 * a format of "[serverRegistrationId]-[other information]" and parse the
 * value as such.
 * <p>
 * The "launch" parameter is intended to be opaque to the client,
 * so this scheme can work. However, any scheme which allows the
 * interceptor to determine which Server Name to return is fine. Other
 * parameters such as "client_id" may be useful here.
 * See <A href="https://smilecdr.com/docs/smart/smart_on_fhir_authorization_flows.html#authorization_code_flow">Authorization Code Flow</A>
 * for an example of parameters which should be available during the
 * initial authorization request by the client.
 */
@Interceptor
public class ServerSelectionInterceptor {

	@CdrHook(CdrPointcut.SMART_FEDERATED_OIDC_PRE_PROVIDER_SELECTION)
	public String selectServer(OidcAuthRequestDetails theAuthRequestDetails) {
		String launchParameter = theAuthRequestDetails.getLaunch();
		if (StringUtils.isNotBlank(launchParameter)) {
			int dashIndex = launchParameter.indexOf('-');
			if (dashIndex != -1) {
				/*
				 * We'll treat the first portion of the launch code as being
				 * a server definition "registration ID". Note that this is not
				 * the same as the server name. Also note that this particular logic
				 * assumes that your registration ID does not have any "-" characters
				 * in it.
				 */
				return launchParameter.substring(0, dashIndex);
			}
		}
		return null;
	}

}
// END SNIPPET: interceptor
